require 'aws-sdk-secretsmanager'
require 'base64'

# class for centralizing Creative's logging logic
class ChiliLogger
  # module responsible for uniformization of values in ChiliLogger
  module Values
    # class to centralize access to all secrets stored in 3rd-party managers
    class Secrets
      # Module to load secrets environment variables
      class AwsSecretsManager
        def initialize(region_name = 'us-east-1')
          @client = Aws::SecretsManager::Client.new(region: region_name)
        end

        def get_secrets_collection(collection_name)
          JSON.parse(get_secret(collection_name))
        end

        private

        # Use this code snippet in your app.
        # If you need more information about configurations or implementing the sample code, visit the AWS docs:
        # https://aws.amazon.com/developers/getting-started/ruby/
        # method from AWS example, ipsis literis
        def get_secret(secret_name)
          # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
          # See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
          # We rethrow the exception by default.
          get_secret_value_response = @client.get_secret_value(secret_id: secret_name)

          # Decrypts secret using the associated KMS CMK.
          # Depending on whether the secret is a string or binary, one of these fields will be populated.
          get_secret_value_response.secret_string || Base64.decode64(get_secret_value_response.secret_binary)
        end
      end
    end
  end
end
