# class for centralizing Creative's logging logic
class ChiliLogger
  # module responsible for uniformization of values in ChiliLogger
  module Values
    # class to enforce consistent typing of values across all classes
    # this is important because of Elasticsearch. Check README, section ##read-before-contributing to know more about it
    class TypeUniformizer
      # responsible for uniformizing log's description primitive types
      class Desc
        def initialize
          @default = ChiliLogger::Values::Default.new
        end

        def desc(desc)
          return {} unless desc.is_a?(Hash)

          uniformized_desc = {}
          valid_desc_attrs.each do |attr_name|
            # only overwrite if key was explicitly set in desc
            next unless desc.key?(attr_name)

            desc_attr = desc[attr_name]
            desc_attr = @default.invalid_string unless valid_desc_attr_formats.include?(desc_attr.class.name)
            desc_attr ||= @default.undefined_string # if type == nil
            uniformized_desc[attr_name] = desc_attr
          end

          uniformized_desc
        end

        def type(type)
          return @default.undefined_string if type.nil?
          return @default.invalid_string unless type.is_a?(String)

          type
        end

        def service(service)
          return @default.undefined_string if service.nil?
          return @default.invalid_string unless service.is_a?(String)

          service
        end

        def action(action)
          return @default.undefined_string if action.nil?
          return @default.invalid_string unless action.is_a?(String)

          action
        end

        private

        def valid_desc_attrs
          %i[type service action]
        end

        def valid_desc_attr_formats
          %w[String NilClass]
        end
      end
    end
  end
end
