# class for centralizing Creative's logging logic
class ChiliLogger
  # module responsible for uniformization of values in ChiliLogger
  module Values
    # class to enforce consistent typing of values across all classes, this is important because of Elasticsearch
    # Check README, section ##read-before-contributing, to know more about it
    class TypeUniformizer
      # responsible for uniformizing the primitive types of a log's main_content
      class MainContent
        def initialize
          @default = ChiliLogger::Values::Default.new
        end

        def main_content(content)
          return {} unless content.is_a?(Hash)

          uniformized_content = {}

          if content.key?(:modified_records) # only overwrite if :modified_records was explicitly set in main_content
            mod_recs = content[:modified_records]
            uniformized_content[:modified_records] = modified_records(mod_recs)
          end
          # only overwrite if :errors was explicitly set in main_content
          uniformized_content[:errors] = errors(content[:errors]) if content.key?(:errors)

          uniformized_content
        end

        def modified_records(records)
          return {} unless records.is_a?(Hash)

          uniformized_recs = {}
          # modified records should be a hash where each key points to an array of valid modified records
          records.each do |key, val|
            val_as_array = val.is_a?(Array) ? val : [].push(val)
            uniformized_val = val_as_array.map { |record| modified_record(record) }
            uniformized_recs[key.to_s] = uniformized_val
          end

          uniformized_recs
        end

        def modified_record(record)
          record.is_a?(Hash) ? record : {}
        end

        def errors(errors)
          return [] unless errors.is_a?(Array)

          errors.map { |err| error(err) }
        end

        # a received error must be either a string or Exception descendent(it will be converted to a string)
        def error(error)
          error = error.inspect if error.class.ancestors.include?(Exception) # stringifies Exception descendents
          error = @default.log_error unless error.is_a?(String)
          error
        end
      end
    end
  end
end
