# class for centralizing Creative's logging logic
class ChiliLogger
  # module responsible for uniformization of values in ChiliLogger
  module Values
    # class to enforce consistent typing of values across all classes
    # this is important because of Elasticsearch. Check README, section ##read-before-contributing to know moreabout it
    class TypeUniformizer
      # responsible for uniformizing log's primitive types
      class User
        def initialize
          @default = ChiliLogger::Values::Default.new
        end

        def user(user)
          return {} unless user.is_a?(Hash)

          uniformized_user = {}
          valid_user_attrs.each do |attr_name|
            # don't enforce typing compliance of attrs that weren't explicitly set in user
            next unless user.key?(attr_name)

            user_attr = user[attr_name]
            user_attr = @default.invalid_string unless valid_user_attr_formats.include?(user_attr.class.name)
            user_attr ||= @default.undefined_string # if type == nil
            uniformized_user[attr_name] = user_attr.to_s
          end

          uniformized_user
        end

        private

        def valid_user_attrs
          %i[cognito_id email company_cognito_id company_name ghost_user_cognito_id]
        end

        def valid_user_attr_formats
          %w[String NilClass Fixnum Bignum]
        end
      end
    end
  end
end
