require 'errors/error_messages/config_error'
require 'brokers/main_broker'
require 'errors/logging_error_handler/logging_error_handler'

# class for centralizing Creative's logging logic
class ChiliLogger
  # class for manipulating unpublished logs that were sent to the fallback_broker
  class UnpublishedLogsManager
    def initialize(msg_broker, logging_error_handler)
      validate_msg_broker_type(msg_broker)
      validate_logging_error_handler_type(logging_error_handler)

      @msg_broker = msg_broker
      @logging_error_handler = logging_error_handler
    end

    def republish
      unpub_logs = @logging_error_handler.fetch_unpublished_logs

      unpub_logs.each do |unpub_log|
        fallback_broker_msg = unpub_log[:fallback_broker_msg]
        log = unpub_log[:log]

        # if a message can't be published by MainBroker, LoggingErrorHandler sends it to the fallback_broker
        # to avoid duplicates in this scenario, we delete the message before even trying to publish it
        @logging_error_handler.delete_unpublished_log(fallback_broker_msg)
        @msg_broker.publish(log)
      end
    end

    private

    def validate_msg_broker_type(msg_broker)
      err_type = ChiliLogger::ConfigError
      err_msg = "#{msg_broker.inspect} should be an instance of ChiliLogger::MainBroker"

      raise(err_type, err_msg) unless msg_broker.class == ChiliLogger::MainBroker
    end

    def validate_logging_error_handler_type(error_handler)
      err_type = ChiliLogger::ConfigError
      err_msg = "#{error_handler} should be an instance of ChiliLogger::LoggingError::Handler"

      raise(err_type, err_msg) unless error_handler.class == ChiliLogger::LoggingErrorHandler
    end
  end
end
